<?php
/**
 * apAuthWifidogV1
 * 
 * This class is the facade class of the authenticator, to be used by the application
 * 
 * 
 * @package    authpuppy
 * @subpackage plugin
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @author 		 Philippe April <philippe@philippeapril.com>
 * @version    BZR: $Id$
 */

class apConnectionPoliciesMain {
  protected static $_plugin; 
  
  /**
   * Returns the plugin object associated with this plugin
   * @return apPlugin
   */
  public static function getPlugin() {
    if (is_null(self::$_plugin))
      self::$_plugin = apPlugin::getPlugin('apNodeExtraPlugin');
    return self::$_plugin;
  }
  
  /**
   * Adds the menu
   */
  public static function getMenu(sfEvent $event, $menus) {
    $menus["Network Policies"] = array();
    $menus["Network Policies"][] = array('text' => 'Manage network policies', 'link' => 'ap_connection_policies_index', 'privilege' => 'admin');
    $menus["Network Policies"][] = array('text' => 'View connection status', 'link' => 'ap_connection_policies_status', 'privilege' => 'Logged');
    return $menus;
  }
  
  public static function initiatingConnection(sfEvent $event, $connection) {
    $policies = Doctrine_Core::getTable('apConnectionPolicies')->getMatchingPolicies($connection);
    $mostrestrictive = array('disconnect_at' => null, 'max_total_data' => null);
    foreach ($policies as $policy) {
      $values = $policy->calculateLimit($connection, $connection->getNodeId());

      // Add this applicable policy
      $applicable = new apApplicablePolicies();
      $applicable->setConnectionId($connection->getId());
      $applicable->setPolicyId($policy->getId());
      $applicable->setTotalData(isset($values['total_data'])?$values['total_data']:0);
      $applicable->setThisTotalData($values['max_total_data']);
      $applicable->setDisconnectAt(!is_null($values['disconnect_at']) ? date('Y-m-d H:i:s', $values['disconnect_at']):null);
      $applicable->save();
      
      if (!is_null($values['disconnect_at'])) {
        // This is the most restrictive policy so far
        if ( is_null($mostrestrictive['disconnect_at']) || ($mostrestrictive['disconnect_at'] > $values['disconnect_at']) ) {
          $mostrestrictive['disconnect_at'] = $values['disconnect_at'];
        }
      }
     if (!is_null($values['max_total_data'])) {
        // This is the most restrictive policy so far
        if (is_null($mostrestrictive['max_total_data']) || ($mostrestrictive['max_total_data'] > $values['max_total_data']) ) {
          $mostrestrictive['max_total_data'] = $values['max_total_data'];
        }
      }
    }
    $connection->setMaxTotalData($mostrestrictive['max_total_data']);
    $connection->setDisconnectAt((!is_null($mostrestrictive['disconnect_at']) ? date('Y-m-d H:i:s', $mostrestrictive['disconnect_at']):null));
    return $connection; 
  }
  
  public static function settingUpConnection(sfEvent $event, $connection) {
    $connection->hasColumn('max_total_data', 'integer', 8, array(
             'default' => 0, 'length' => 8,
        ));
    $connection->hasColumn('disconnect_at', 'timestamp');
    $connection->hasMany('apApplicablePolicies as Policies', array(
             'local' => 'id',
             'foreign' => 'connection_id'));
            
    return $connection; 
  }
  
  public static function verifyConnection(sfEvent $event, $retvals) {
    $connection = $event['connection'];
    if (!is_null($connection)) {
        if ( !is_null($connection->getMaxTotalData()) && ($connection->getIncoming() + $connection->getOutgoing()) > $connection->getMaxTotalData()) {
          $retvals['auth'] = 0; 
          $retvals['messages'] = "| You've used all your bandwidth for this session.";
        }
        if ( !is_null($connection->getDisconnectAt()) && (time() > strtotime($connection->getDisconnectAt()))) {
          $retvals['auth'] = 0; 
          $retvals['messages'] = "| Your connection has expired.";
        }
    }
    return $retvals;
  }
  
  public static function displayStatus(sfEvent $event, $retvals) {
    $identity = $event['identity'];
    if (!is_null($identity)) {
      $policies = Doctrine::getTable('apApplicablePolicies');
      $status = $policies->getForIdentity($identity);
      foreach ($status as $s) {
        $retvals[] = $s;
      }
    }
    return $retvals;
  }
  
}
